@@ -94,36 +94,30 @@ static void
 print_devnode(gpointer data, gpointer user_data)
 {
 	const gchar *devnode = data;
-	gchar *name = NULL;
+	g_autofree gchar *name = NULL;
 	gsize size;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
-	gchar *basename = g_path_get_basename(devnode);
-	char *path = g_strdup_printf("/sys/class/input/%s/device/name", basename);
+	g_autofree gchar *basename = g_path_get_basename(devnode);
+	g_autofree char *path = g_strdup_printf("/sys/class/input/%s/device/name", basename);
 
-	g_free(basename);
 	if (g_file_get_contents(path, &name, &size, &error)) {
-		printf("  - %s: '%.*s'\n", devnode, (int)(size - 1), name);
+		printf("      - %s: '%.*s'\n", devnode, (int)(size - 1), name);
 	} else {
 		fprintf(stderr, "%s\n", error->message);
 	}
-	if (error)
-		g_error_free(error);
-	g_free(name);
-	g_free(path);
 }
 
 static void
 tablet_print_yaml(gpointer data, gpointer user_data)
 {
-	WacomDeviceDatabase *db = user_data;
 	struct tablet *d = data;
 	const char *name = libwacom_get_name(d->dev);
 	const char *bus = "unknown";
 	int vid = libwacom_get_vendor_id(d->dev);
 	int pid = libwacom_get_product_id(d->dev);
 	WacomBusType bustype = libwacom_get_bustype(d->dev);
-	const int *styli;
+	g_autofree const WacomStylus **styli;
 	int nstyli;
 
 	switch (bustype) {
@@ -135,22 +129,26 @@ tablet_print_yaml(gpointer data, gpointer user_data)
 			break;
 	}
 
-	printf("- name: '%s'\n", name);
-	printf("  bus: '%s'\n", bus);
-	printf("  vid: '0x%04x'\n", vid);
-	printf("  pid: '0x%04x'\n", pid);
-	printf("  nodes: \n");
+	printf("  - name: '%s'\n", name);
+	printf("    bus: '%s'\n", bus);
+	printf("    vid: 0x%04x\n", vid);
+	printf("    pid: 0x%04x\n", pid);
+	printf("    nodes: \n");
 	g_list_foreach(d->nodes, print_devnode, NULL);
 
-	styli = libwacom_get_supported_styli(d->dev, &nstyli);
-	printf("  styli:%s\n", nstyli > 0 ? "" : "[]");
+	styli = libwacom_get_styli(d->dev, &nstyli);
+	printf("    styli:%s\n", nstyli > 0 ? "" : " []");
 	for (int i = 0; i < nstyli; i++) {
-		const WacomStylus *stylus = libwacom_stylus_get_for_id(db, styli[i]);
+		const WacomStylus *stylus = styli[i];
 		const char *type = "invalid";
 		WacomAxisTypeFlags axes = libwacom_stylus_get_axes(stylus);
 		WacomEraserType eraser_type = libwacom_stylus_get_eraser_type(stylus);
 		const char *etype= "unknown";
 
+		/* warning: Value stored to 'type' during its initialization is never read [deadcode.DeadStores] */
+		(void)type;
+		(void)etype;
+
 		switch (libwacom_stylus_get_type(stylus)) {
 		case WSTYLUS_UNKNOWN:  type = "unknown"; break;
 		case WSTYLUS_GENERAL:  type = "general"; break;
@@ -163,16 +161,17 @@ tablet_print_yaml(gpointer data, gpointer user_data)
 		case WSTYLUS_3D:       type = "3D"; break;
 		case WSTYLUS_MOBILE:   type = "mobile"; break;
 		}
-		printf("   - id: 0x%x\n", libwacom_stylus_get_id(stylus));
-		printf("     name: '%s'\n", libwacom_stylus_get_name(stylus));
-		printf("     type: '%s'\n", type);
-		printf("     axes: ['x', 'y' %s%s%s%s%s]\n",
+		printf("      - id: 0x%x\n", libwacom_stylus_get_id(stylus));
+		printf("        vid: 0x%04x\n", libwacom_stylus_get_vendor_id(stylus));
+		printf("        name: '%s'\n", libwacom_stylus_get_name(stylus));
+		printf("        type: '%s'\n", type);
+		printf("        axes: ['x', 'y' %s%s%s%s%s]\n",
 			axes & WACOM_AXIS_TYPE_TILT ? ", 'tilt'" : "",
 			axes & WACOM_AXIS_TYPE_ROTATION_Z ? ", 'rotation'" : "",
 			axes & WACOM_AXIS_TYPE_DISTANCE ? ", 'distance'" : "",
 			axes & WACOM_AXIS_TYPE_PRESSURE ? ", 'pressure'" : "",
 			axes & WACOM_AXIS_TYPE_SLIDER ? ", 'slider'" : "");
-		printf("     buttons: %d\n", libwacom_stylus_get_num_buttons(stylus));
+		printf("        buttons: %d\n", libwacom_stylus_get_num_buttons(stylus));
 
 		switch (eraser_type) {
 		case WACOM_ERASER_UNKNOWN: etype = "unknown"; break;
@@ -182,15 +181,17 @@ tablet_print_yaml(gpointer data, gpointer user_data)
 		}
 
 		if (libwacom_stylus_is_eraser(stylus)) {
-			printf("     is_eraser: 'true'\n");
-			printf("     eraser_type: '%s'\n", etype);
+			printf("        is_eraser: 'true'\n");
+			printf("        eraser_type: '%s'\n", etype);
 		} else {
-			printf("     erasers: [");
+			printf("        erasers: [");
 			if (libwacom_stylus_has_eraser(stylus)) {
 				int npaired;
-				const int *paired = libwacom_stylus_get_paired_ids(stylus, &npaired);
-				for (int j = 0; j < npaired; j++)
-					printf("%s0x%x", j == 0 ? "" : ", ", paired[j]);
+				g_autofree const WacomStylus **paired = libwacom_stylus_get_paired_styli(stylus, &npaired);
+				for (int j = 0; j < npaired; j++) {
+					const WacomStylus *p = paired[j];
+					printf("%s0x%x", j == 0 ? "" : ", ", libwacom_stylus_get_id(p));
+				}
 			}
 			printf("]\n");
 		}
@@ -201,20 +202,29 @@ tablet_print_yaml(gpointer data, gpointer user_data)
 static void
 check_if_udev_tablet(const char *path)
 {
-	GUdevClient *client;
-	GUdevDevice *device;
 	const char * const subsystems[] = { "input", NULL };
+	g_autoptr(GUdevClient) client = g_udev_client_new (subsystems);
+	g_autoptr(GUdevDevice) device = g_udev_client_query_by_device_file (client, path);
 
-	client = g_udev_client_new (subsystems);
-	device = g_udev_client_query_by_device_file (client, path);
 	if (device &&
 	    g_udev_device_get_property_as_boolean (device, "ID_INPUT_TABLET")) {
+		g_autoptr(GUdevDevice) parent = g_udev_device_get_parent (device);
+		g_autofree char *info = NULL;
+
+		if (parent) {
+			const char *bus = g_udev_device_get_property (parent, "ID_BUS");
+			const char *vid = g_udev_device_get_property (parent, "ID_VENDOR_ID");
+			const char *pid = g_udev_device_get_property (parent, "ID_MODEL_ID");
+			const char *name = g_udev_device_get_property (parent, "NAME");
+
+			if (bus && vid && pid && name)
+				info = g_strdup_printf("(%s:%s:%s - %s) ", bus, vid, pid, name);
+		}
+
 		fprintf(stderr,
-			"%s is a tablet but not supported by libwacom\n",
-			path);
+			"%s %sis a tablet but not supported by libwacom\n",
+			path, info ? info : "");
 	}
-	g_object_unref (device);
-	g_object_unref (client);
 }
 
 static gboolean
@@ -238,22 +248,19 @@ static GOptionEntry opts[] = {
 int main(int argc, char **argv)
 {
 	WacomDeviceDatabase *db;
-	GOptionContext *context;
-	GError *error;
+	g_autoptr(GOptionContext) context = g_option_context_new (NULL);
+	g_autoptr(GError) error = NULL;
 	GList *tabletlist = NULL;
 	GDir *dir = NULL;
 	const char *filename;
 
-	context = g_option_context_new (NULL);
+
 
 	g_option_context_add_main_entries (context, opts, NULL);
-	error = NULL;
 
 	if (!g_option_context_parse (context, &argc, &argv, &error)) {
-		if (error != NULL) {
+		if (error != NULL)
 			fprintf (stderr, "%s\n", error->message);
-			g_error_free (error);
-		}
 		return EXIT_FAILURE;
 	}
 
@@ -278,7 +285,6 @@ int main(int argc, char **argv)
 	dir = g_dir_open("/dev/input", 0, &error);
 	if (!dir) {
 		fprintf(stderr, "%s\n", error->message);
-		g_error_free(error);
 		return EXIT_FAILURE;
 	}
 
